package com.example;

import com.example.entity.ProductEntity;
import com.example.entity.OrderEntity;
import com.example.repository.MonthlyProductStockRepository;
import com.example.repository.ReceiptShipmentRepository;
import com.example.repository.OrderRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.batch.item.ItemProcessor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

@Component
public class ProductItemProcessor implements ItemProcessor<ProductEntity, OrderEntity> {

    private static final Logger logger = LoggerFactory.getLogger(ProductItemProcessor.class);

    @Autowired
    private MonthlyProductStockRepository monthlyProductStockRepository;

    @Autowired
    private ReceiptShipmentRepository receiptShipmentRepository;

    @Autowired
    private OrderRepository orderRepository;

    @Override
    public OrderEntity process(ProductEntity product) throws Exception {
        logger.info("処理開始: 商品ID = {}", product.getId());

        // 安全在庫の確認
        BigDecimal initialStock = monthlyProductStockRepository.findQuantityByProductIdAndCurrentMonth(product.getId())
                .orElse(BigDecimal.ZERO);
        logger.info("月初商品在庫テーブルの数量取得: 商品ID = {}", product.getId());

        LocalDateTime startOfMonth = LocalDateTime.parse(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMM")) + "01T00:00:00");
        LocalDateTime currentDate = LocalDateTime.now();

        BigDecimal incomingQuantity = receiptShipmentRepository.findTotalIncomingQuantityByProductId(product.getId(), startOfMonth, currentDate)
                .orElse(BigDecimal.ZERO);
        logger.info("入荷数量算出: 商品ID = {}", product.getId());

        BigDecimal outgoingQuantity = receiptShipmentRepository.findTotalOutgoingQuantityByProductId(product.getId(), startOfMonth, currentDate)
                .orElse(BigDecimal.ZERO);
        logger.info("出荷数量算出: 商品ID = {}", product.getId());

        BigDecimal currentStock = initialStock.add(incomingQuantity).subtract(outgoingQuantity);
        logger.info("現時点の理論在庫算出: 商品ID = {}", product.getId());

        if (product.getSafetyStockOfNumber().compareTo(currentStock) > 0) {
            if (product.getInventoryLevel() == 1) {
                // 承認フローの確認と実行
                logger.info("(comment Start)共通承認フロー存在確認(comment End)");
                logger.info("(comment Start)共通承認フロー回付(comment End)");
            } else if (product.getInventoryLevel() == 2) {
                // 自動発注の確認とデータ作成
                if (!orderRepository.findActiveOrderByProductId(product.getId()).isPresent()) {
                    logger.info("発注データ作成: 商品ID = {}", product.getId());
                    OrderEntity order = new OrderEntity();
                    order.setProductId(product.getId());
                    order.setOrderQuantity(product.getAutomaticOrderUnit());
                    order.setOrderDate(LocalDateTime.now());
                    order.setOrderStatus(1);
                    order.setSupplierId(product.getVendorId());
                    logger.info("処理終了: 商品ID = {}", product.getId());
                    return order;
                }
            }
        }

        logger.info("処理終了: 商品ID = {}", product.getId());
        return null;
    }
}